package com.clx.performance.service.impl.vehiclewarn;

import cn.hutool.json.JSONObject;
import com.alibaba.fastjson.JSON;
import com.clx.message.feign.ClxMessageOpenapiFeign;
import com.clx.message.req.batch.AliBatchSmsMessageReq;
import com.clx.message.req.message.ClinkMessageReq;
import com.clx.performance.config.ClxMessageConfig;
import com.clx.performance.dao.OrderChildDao;
import com.clx.performance.dao.OrderGoodsDao;
import com.clx.performance.dao.vehiclewarn.VehicleWarnChildDao;
import com.clx.performance.dao.vehiclewarn.VehicleWarnInfoDao;
import com.clx.performance.dao.vehiclewarn.VehicleWarnUserDao;
import com.clx.performance.dto.gd.GdRouteDTO;
import com.clx.performance.enums.OrderChildEnum;
import com.clx.performance.enums.ResultEnum;
import com.clx.performance.model.OrderChild;
import com.clx.performance.model.OrderGoods;
import com.clx.performance.model.vehiclewarn.VehicleWarnChild;
import com.clx.performance.model.vehiclewarn.VehicleWarnInfo;
import com.clx.performance.model.vehiclewarn.VehicleWarnUser;
import com.clx.performance.param.mq.vehiclewarn.VehicleWarnChildAddMqParam;
import com.clx.performance.param.mq.vehiclewarn.VehicleWarnTruckWarnMqParam;
import com.clx.performance.param.mq.vehiclewarn.VehicleWarnTruckWarnNoticeMqParam;
import com.clx.performance.service.trace.TruckTraceService;
import com.clx.performance.service.vehiclewarn.VehicleWarnLoadAndUnloadWarnService;
import com.clx.performance.service.vehiclewarn.VehicleWarnMqHandlerService;
import com.clx.performance.service.vehiclewarn.VehicleWarnSendAddressWarnService;
import com.clx.performance.utils.gd.GdService;
import com.msl.common.exception.ServiceSystemException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

@Slf4j
@Service
public class VehicleWarnMqHandlerServiceImpl implements VehicleWarnMqHandlerService {
    private static final Integer SEC_FER_MIN = 60;

    @Autowired
    private OrderChildDao orderChildDao;
    @Autowired
    private VehicleWarnChildDao vehicleWarnChildDao;
    @Autowired
    private VehicleWarnInfoDao vehicleWarnInfoDao;
    @Autowired
    private OrderGoodsDao orderGoodsDao;
    @Autowired
    private VehicleWarnUserDao vehicleWarnUserDao;
    @Autowired
    private VehicleWarnSendAddressWarnService vehicleWarnSendAddressWarnService;
    @Autowired
    private VehicleWarnLoadAndUnloadWarnService vehicleWarnLoadAndUnloadWarnService;
    @Autowired
    private TruckTraceService truckTraceService;
    @Autowired
    private GdService gdService;
    @Autowired
    private ClxMessageConfig clxMessageConfig;
    @Autowired
    private ClxMessageOpenapiFeign clxMessageOpenapiFeign;

    @Override
    public void orderChildAdd(VehicleWarnChildAddMqParam mq) {
        OrderChild orderChild = orderChildDao.getByChildNo(mq.getChildNo()).orElseThrow(ResultEnum.DATA_NOT_FIND);
        OrderGoods orderGoods = orderGoodsDao.getByOrderGoodsNo(orderChild.getOrderGoodsNo()).orElseThrow(ResultEnum.DATA_NOT_FIND);

        int arriveSendExpectTime = getNeedTime(orderChild.getTruckNo(), orderChild.getDriverUserNo(), orderGoods.getSendLongitude(), orderGoods.getSendLatitude());
        int arriveReceiveExpectTime = getNeedTime(orderChild.getTruckNo(), orderChild.getDriverUserNo(), orderGoods.getReceiveLongitude(), orderGoods.getReceiveLatitude());

        VehicleWarnChild vehicleWarnChild = new VehicleWarnChild();
        vehicleWarnChild.setOrderNo(orderChild.getOrderNo());
        vehicleWarnChild.setChildNo(orderChild.getChildNo());
        vehicleWarnChild.setTruckNo(orderChild.getTruckNo());
        vehicleWarnChild.setDriverUserNo(orderChild.getDriverUserNo());
        vehicleWarnChild.setSendLongitude(orderGoods.getSendLongitude());
        vehicleWarnChild.setSendLatitude(orderGoods.getSendLatitude());
        vehicleWarnChild.setReceiveLongitude(orderGoods.getReceiveLongitude());
        vehicleWarnChild.setReceiveLatitude(orderGoods.getReceiveLatitude());
        vehicleWarnChild.setArriveSendExpectTime(arriveSendExpectTime==0? null : arriveSendExpectTime);
        vehicleWarnChild.setArriveReceiveExpectTime(arriveReceiveExpectTime==0? null : arriveReceiveExpectTime);

        vehicleWarnChild.setTraceTime(orderChild.getPayTime());

        vehicleWarnChildDao.saveEntity(vehicleWarnChild);
    }

    @Override
    public void truckWarn(VehicleWarnTruckWarnMqParam mq) {
        OrderChild orderChild = orderChildDao.getByChildNo(mq.getChildNo()).orElseThrow(ResultEnum.DATA_NOT_FIND);

        VehicleWarnChild vehicleWarnChild = vehicleWarnChildDao.findByChildNo(mq.getChildNo()).orElseThrow(ResultEnum.DATA_NOT_FIND);

        // 前往货源地超时
        if (Objects.equals(orderChild.getStatus(), OrderChildEnum.Status.CREATED.getCode())
                || Objects.equals(orderChild.getStatus(), OrderChildEnum.Status.PAY.getCode())
                || Objects.equals(orderChild.getStatus(), OrderChildEnum.Status.GO_TO_SEND.getCode())
        ){
            vehicleWarnSendAddressWarnService.gotoSendAddressTimeout(orderChild, vehicleWarnChild);
        }

        // 到达货源地超时
        if (Objects.equals(orderChild.getStatus(), OrderChildEnum.Status.CREATED.getCode())
                || Objects.equals(orderChild.getStatus(), OrderChildEnum.Status.PAY.getCode())
                || Objects.equals(orderChild.getStatus(), OrderChildEnum.Status.GO_TO_SEND.getCode())
        ){
            vehicleWarnSendAddressWarnService.arriveSendAddressTimeout(orderChild, vehicleWarnChild);
        }

        if (Objects.equals(orderChild.getStatus(), OrderChildEnum.Status.ARRIVE_SEND.getCode())
        ){
            vehicleWarnLoadAndUnloadWarnService.loadTimeout(orderChild, vehicleWarnChild);
        }
        if (Objects.equals(orderChild.getStatus(), OrderChildEnum.Status.ARRIVE_RECEIVE.getCode())
        ){
            vehicleWarnLoadAndUnloadWarnService.unloadTimeout(orderChild, vehicleWarnChild);
        }

    }

    @Override
    public void truckWarnNotice(VehicleWarnTruckWarnNoticeMqParam mq) {
        VehicleWarnInfo vehicleWarnInfo = vehicleWarnInfoDao.findById(mq.getInfoId()).orElseThrow(ResultEnum.DATA_NOT_FIND);

        List<String> mobileList = new ArrayList<>();

        // 过滤需要通知的人员
        List<VehicleWarnUser> list = vehicleWarnUserDao.listAllEnable();
        for (VehicleWarnUser item : list) {
            List<Integer> warnTypeList = JSON.parseArray(item.getWarmTypeJson(),Integer.class);
            if (warnTypeList.contains(vehicleWarnInfo.getWarnType())){
                mobileList.add(item.getMobile());
            }
        }

        if (mobileList.isEmpty()) {return;}
        log.info("车辆预警通知, id:{}, mobileList:{}", vehicleWarnInfo.getId(), JSON.toJSONString(mobileList));

        if (vehicleWarnInfo.getWarnLevel() == 2){   //2级短信
            sendSms(mobileList, vehicleWarnInfo.getWarnName(), vehicleWarnInfo.getTruckNo(), vehicleWarnInfo.getChildNo());
        }
        else if (vehicleWarnInfo.getWarnLevel() == 3) {   //3级外呼
            sendClink(mobileList, vehicleWarnInfo.getWarnName(), vehicleWarnInfo.getTruckNo(), vehicleWarnInfo.getChildNo());
        }

    }

    /**
     * 获取需要时间（min）
     */
    private int getNeedTime(String truckNo, Long userNo,
                            BigDecimal destinationLongitude, BigDecimal destinationLatitude){
        int needTime = 0;
        BigDecimal[] lastPosition = truckTraceService.getLastPosition(truckNo, userNo);
        if (lastPosition != null) {
            try {
                List<GdRouteDTO> gdRouteDTOS = gdService.getRoute(lastPosition[0], lastPosition[1], destinationLongitude, destinationLatitude);

                if (!gdRouteDTOS.isEmpty()){
                    needTime = gdRouteDTOS.get(0).getDuration() /SEC_FER_MIN;
                }

            } catch (ServiceSystemException e) {
                log.info("获取高德线路失败, truckNo:{}, userNo:{}, msg:{}", truckNo, userNo, e.getMessage());
            }
        }

        return needTime;
    }

    /**
     * 发送预警短信
     */
    @Override
    public String sendSms(List<String> mobileList, String warnName, String truckNo, String childNo) {
        //触发${warnName}预警；车牌号${truckNo}；运单${childNo}；请尽快核实情况
        AliBatchSmsMessageReq req = new AliBatchSmsMessageReq();
        JSONObject jsonObject = new JSONObject();
        jsonObject.set("warnName", warnName);
        jsonObject.set("truckNo", truckNo);
        jsonObject.set("childNo", childNo);

        req.setTemplateCode(clxMessageConfig.getVehicleWarnTemplateCode());
        req.setChannelId(clxMessageConfig.getChannelId());
        req.setAppId(clxMessageConfig.getAppId().toString());
        req.setMobileList(mobileList);
        req.setContent(jsonObject.toString());
        clxMessageOpenapiFeign.sendAliSmsBatch(req);

        return UUID.randomUUID().toString();
    }

    /**
     * 发送预警外呼
     */
    @Override
    public String sendClink(List<String> mobileList, String warnName, String truckNo, String childNo) {
        //触发{指定预警名称}预警；车牌号{车牌号}；运单{运单号}；请尽快核实情况
        ClinkMessageReq req = new ClinkMessageReq();

        StringBuilder sb = new StringBuilder();
        sb.append("触发");
        sb.append(warnName);
        sb.append("预警；车牌号");
        sb.append(truckNo);
        sb.append("；运单");
        sb.append(childNo);
        sb.append("；请尽快核实情况");

        String content = sb.toString();

        req.setChannelId(clxMessageConfig.getVehicleWarnChannelId());
        req.setAppId(clxMessageConfig.getAppId().toString());
        req.setContent(content);

        for (String mobile : mobileList) {
            req.setMobile(mobile);
            clxMessageOpenapiFeign.sendClink(req);
        }

        return UUID.randomUUID().toString();
    }

}
